@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2005 Double Precision, Inc.
+** Copyright 1998 - 2007 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -52,7 +52,7 @@
 
 #include	<netdb.h>
 
-static const char rcsid[]="$Id: tcpd.c,v 1.41 2005/07/02 01:13:57 mrsam Exp $";
+static const char rcsid[]="$Id: tcpd.c,v 1.45 2007/08/30 01:38:49 mrsam Exp $";
 
 static const char *accessarg=0;
 static const char *accesslocal=0;
@@ -1051,12 +1051,13 @@ static int doit(int argn, int argc, char **argv)
 	}
 #endif
 
+	signal(SIGPIPE, SIG_IGN);
 	for (;;)
 	{
 		int n;
 		int sockfd;
 		RFC1035_NETADDR	sin;
-		int	sinl;
+		socklen_t	sinl;
 
 		fdr=fdrcopy;
 
@@ -1177,7 +1178,7 @@ static void accepted(int n, int sockfd, RFC1035_NETADDR *sin, int sinl,
 		RFC1035_NETADDR lsin;
 		RFC1035_ADDR laddr;
 		int	lport;
-		int	i=sizeof(lsin);
+		socklen_t	i=sizeof(lsin);
 		
 			if (sox_getsockname(sockfd, (struct sockaddr *)&lsin, &i) == 0 &&
 				rfc1035_sockaddrip(&lsin, i, &laddr) == 0 &&
@@ -1192,8 +1193,12 @@ static void accepted(int n, int sockfd, RFC1035_NETADDR *sin, int sinl,
 		if (allowaccess(&addr,0) == 0)
 		{
 			if (denymsgarg) {
-				write(sockfd, denymsgarg, strlen(denymsgarg));
-				write(sockfd, "\n", 1);
+				if (write(sockfd, denymsgarg, strlen(denymsgarg)) < 0 ||
+				    write(sockfd, "\n", 1) < 0)
+				{
+					sox_close(sockfd);
+					_exit(1);
+				}
 			}
 			sox_close(sockfd);
 			_exit(0);
@@ -1588,11 +1593,12 @@ RFC1035_NETADDR lsin;
 RFC1035_ADDR laddr;
 int	lport;
 
-int	i;
+socklen_t	i;
 int	ipcnt, ccnt;
 char	buf[RFC1035_MAXNAMESIZE+128];
 struct blocklist_s *bl;
 const char *remoteinfo;
+const char *p;
 
 	i=sizeof(lsin);
 	if (sox_getsockname(fd, (struct sockaddr *)&lsin, &i) ||
@@ -1619,6 +1625,15 @@ const char *remoteinfo;
 		putenv(q);
 	}
 
+/* check if it's an exception to the global ip limit */
+	if( (p=getenv("MAXCPERIP")) != NULL )
+	{
+		int j = atoi(p);
+
+		if( j > 0 )
+			maxperip = j;
+	}
+
 	for (i=0, ipcnt=ccnt=0; i<nprocs; i++)
 	{
 	RFC1035_ADDR *psin;
@@ -1634,7 +1649,11 @@ const char *remoteinfo;
 
 		if (j >= sizeof(*addr) &&
 			++ipcnt >= maxperip)
+		{
+			rfc1035_ntoa(addr, buf);
+			fprintf(stderr,"ALERT: Maximum connection limit reached for %s\n",buf);
 			_exit(0);	/* Too many from same IP address */
+		}
 
 		if ( j >= sizeof(*addr)-1 &&
 			++ccnt >= maxperc)
@@ -1668,6 +1687,8 @@ const char *remoteinfo;
 		sox_dup(1);
 	}
 	proxy();
+	signal(SIGPIPE, SIG_DFL);
+
 	execv(prog, argv);
 	perror(prog);
 	exit(1);
